@@ -6,7 +6,7 @@ Generic Python packager for Android / iOS. Desktop later.
 
 '''
 
-__version__ = '1.4.0'
+__version__ = '1.5.0'
 
 import os
 import re
@@ -26,6 +26,8 @@ from shutil import copyfile, rmtree, copytree, move
 from fnmatch import fnmatch
 
 from pprint import pformat
+import shlex
+import pexpect
 
 from urllib.request import FancyURLopener
 from configparser import ConfigParser
@@ -157,7 +159,7 @@ class Buildozer:
     def prepare_for_build(self):
         '''Prepare the build.
         '''
-        assert(self.target is not None)
+        assert self.target is not None
         if hasattr(self.target, '_build_prepared'):
             return
 
@@ -188,8 +190,8 @@ class Buildozer:
 
         (:meth:`prepare_for_build` must have been call before.)
         '''
-        assert(self.target is not None)
-        assert(hasattr(self.target, '_build_prepared'))
+        assert self.target is not None
+        assert hasattr(self.target, '_build_prepared')
 
         if hasattr(self.target, '_build_done'):
             return
@@ -261,7 +263,6 @@ class Buildozer:
         kwargs.setdefault('stdout', PIPE)
         kwargs.setdefault('stderr', PIPE)
         kwargs.setdefault('close_fds', True)
-        kwargs.setdefault('shell', True)
         kwargs.setdefault('show_output', self.log_level > 1)
 
         show_output = kwargs.pop('show_output')
@@ -357,7 +358,6 @@ class Buildozer:
                 process.returncode)
 
     def cmd_expect(self, command, **kwargs):
-        from pexpect import spawnu
 
         # prepare the environ, based on the system + our own env
         env = environ.copy()
@@ -378,7 +378,7 @@ class Buildozer:
             self.debug('Run (expect) {0!r} ...'.format(command.split()[0]))
 
         self.debug('Cwd {}'.format(kwargs.get('cwd')))
-        return spawnu(command, **kwargs)
+        return pexpect.spawnu(shlex.join(command), **kwargs)
 
     def check_configuration_tokens(self):
         '''Ensure the spec file is 'correct'.
@@ -410,10 +410,10 @@ class Buildozer:
             adderror('[app] "version.filename" is missing'
                      ', required by "version.regex"')
 
-        orientation = get('app', 'orientation', 'landscape')
-        if orientation not in ('landscape', 'portrait', 'all', 'sensorLandscape'):
-            adderror('[app] "orientation" have an invalid value')
-
+        orientation = self.config.getlist("app", "orientation", ["landscape"])
+        for o in orientation:
+            if o not in ("landscape", "portrait", "landscape-reverse", "portrait-reverse"):
+                adderror(f'[app] "{o}" is not a valid  value for "orientation"')
         if errors:
             self.error('{0} error(s) found in the buildozer.spec'.format(
                 len(errors)))
@@ -515,9 +515,11 @@ class Buildozer:
     def _install_application_requirement(self, module):
         self._ensure_virtualenv()
         self.debug('Install requirement {} in virtualenv'.format(module))
-        self.cmd('pip install --target={} {}'.format(self.applibs_dir, module),
-                 env=self.env_venv,
-                 cwd=self.buildozer_dir)
+        self.cmd(
+            ["pip", "install", f"--target={self.applibs_dir}", module],
+            env=self.env_venv,
+            cwd=self.buildozer_dir,
+        )
 
     def check_garden_requirements(self):
         garden_requirements = self.config.getlist('app',
@@ -530,13 +532,15 @@ class Buildozer:
             return
         self.venv = join(self.buildozer_dir, 'venv')
         if not self.file_exists(self.venv):
-            self.cmd('python3 -m venv ./venv',
+            self.cmd(["python3", "-m", "venv", "./venv"],
                     cwd=self.buildozer_dir)
 
         # read virtualenv output and parse it
-        output = self.cmd('bash -c "source venv/bin/activate && env"',
-                get_stdout=True,
-                cwd=self.buildozer_dir)
+        output = self.cmd(
+            ["bash", "-c", "source venv/bin/activate && env"],
+            get_stdout=True,
+            cwd=self.buildozer_dir,
+        )
         self.env_venv = copy(self.environ)
         for line in output[0].splitlines():
             args = line.split('=', 1)
@@ -594,22 +598,22 @@ class Buildozer:
 
     def file_extract(self, archive, cwd=None):
         if archive.endswith('.tgz') or archive.endswith('.tar.gz'):
-            self.cmd('tar xzf {0}'.format(archive), cwd=cwd)
+            self.cmd(["tar", "xzf", archive], cwd=cwd)
             return
 
         if archive.endswith('.tbz2') or archive.endswith('.tar.bz2'):
             # XXX same as before
-            self.cmd('tar xjf {0}'.format(archive), cwd=cwd)
+            self.cmd(["tar", "xjf", archive], cwd=cwd)
             return
 
         if archive.endswith('.bin'):
             # To process the bin files for linux and darwin systems
-            self.cmd('chmod a+x {0}'.format(archive), cwd=cwd)
-            self.cmd('./{0}'.format(archive), cwd=cwd)
+            self.cmd(["chmod", "a+x", archive], cwd=cwd)
+            self.cmd([f"./{archive}"], cwd=cwd)
             return
 
         if archive.endswith('.zip'):
-            self.cmd('unzip -q {}'.format(join(cwd, archive)), cwd=cwd)
+            self.cmd(["unzip", "-q", join(cwd, archive)], cwd=cwd)
             return
 
         raise Exception('Unhandled extraction for type {0}'.format(archive))
